package com.jeffknecht.dynadao.impl;

import org.springframework.jdbc.core.namedparam.AbstractSqlParameterSource;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;

/**
 * Encapsulates an object as an SqlParameterSource and extends the
 * source by allowing additional values to be set.  Note that the
 * bean values always take precedence in the case of a name
 * conflict.
 * 
 * @author jeff.knecht
 *
 */
public class AugmentedBeanParameterSource extends AbstractSqlParameterSource {
	MapSqlParameterSource internalMap;
	BeanPropertySqlParameterSource internalBean;
	
	/**
	 * Constructor.
	 * 
	 * @param object The bean to use as the primary parameter source
	 */
	public AugmentedBeanParameterSource(Object object) {
		this.internalBean = new BeanPropertySqlParameterSource(object);
		this.internalMap = new MapSqlParameterSource();
	}

	/**
	 * Indicates whether parameter name exists.
	 */
	public boolean hasValue(String paramName) {
		return internalBean.hasValue(paramName) || internalMap.hasValue(paramName);
	}

	/**
	 * Returns the value associated with the supplied parameter name.
	 */
	public Object getValue(String paramName) throws IllegalArgumentException {
		if (internalBean.hasValue(paramName)) {
			return internalBean.getValue(paramName);
		}
		
		if (internalMap.hasValue(paramName)) {
			return internalMap.getValue(paramName);
		}
		
		throw new IllegalArgumentException("No value registered for key '" + paramName + "'");
	}

	/**
	 * Adds the specified value.
	 * 
	 * @param paramName
	 * @param value
	 * @return
	 */
	public AugmentedBeanParameterSource addValue(String paramName, Object value) {
		internalMap.addValue(paramName, value);
		return this;
	}

	/**
	 * Adds the specified value.
	 * 
	 * @param paramName
	 * @param value
	 * @param sqlType
	 * @return
	 */
	public AugmentedBeanParameterSource addValue(String paramName, Object value, int sqlType) {
		internalMap.addValue(paramName, value, sqlType);
		return this;
	}

	/**
	 * Adds the specified value.
	 * 
	 * @param paramName
	 * @param value
	 * @param sqlType
	 * @param typeName
	 * @return
	 */
	public AugmentedBeanParameterSource addValue(String paramName, Object value, int sqlType, String typeName) {
		internalMap.addValue(paramName, value, sqlType, typeName);
		return this;
	}

}
